@@ -1,22 +1,21 @@
 (* camlp4r ./pa_html.cmo *)
-(* $Id: birthday.ml,v 4.11 2004/12/14 09:30:10 ddr Exp $ *)
-(* Copyright (c) 1998-2005 INRIA *)
+(* $Id: birthday.ml,v 5.16 2007/01/19 01:53:16 ddr Exp $ *)
+(* Copyright (c) 1998-2007 INRIA *)
 
-open Def;
 open Config;
+open Def;
+open Gwdb;
+open Hutil;
 open Util;
-open Gutil;
 
 type date_event = [ DeBirth | DeDeath of death_reason ];
 
 value print_anniversary_day conf base dead_people liste =
-  do {
-    Wserver.wprint "<ul>\n";
+  tag "ul" begin
     List.iter
       (fun (p, a, date_event, txt_of) ->
-         let is = index_of_sex p.sex in
-         do {
-           html_li conf;
+         let is = index_of_sex (get_sex p) in
+         tag "li" begin
            Wserver.wprint "%s\n" (txt_of conf base p);
            if not dead_people then Wserver.wprint " <em>%d</em>\n" a
            else
@@ -32,10 +31,9 @@ value print_anniversary_day conf base dead_people liste =
              in
              Wserver.wprint ", <em>%s %s %d</em>\n" txt
                (transl conf "in (year)") a;
-         })
+         end)
       liste;
-    Wserver.wprint "</ul>\n";
-  }
+  end
 ;
 
 value gen_print conf base mois f_scan dead_people =
@@ -46,14 +44,14 @@ value gen_print conf base mois f_scan dead_people =
       else transl conf "birthdays"
     in
     Wserver.wprint "%s %s" (capitale lab)
-      (nominative (transl_nth conf "(month)" (mois - 1)))
+      (Util.translate_eval (transl_nth conf "(month)" (mois - 1)))
   in
   do {
     try
       while True do {
         let (p, txt_of) = f_scan () in
         if not dead_people then
-          match (Adef.od_of_codate p.birth, p.death) with
+          match (Adef.od_of_codate (get_birth p), get_death p) with
           [ (Some (Dgreg d _), NotDead | DontKnowIfDead) ->
               if d.prec = Sure && d.day <> 0 && d.month <> 0 &&
                  d.month = mois && d.delta = 0 then
@@ -65,11 +63,11 @@ value gen_print conf base mois f_scan dead_people =
               else ()
           | _ -> () ]
         else
-          match p.death with
+          match get_death p with
           [ NotDead | DontKnowIfDead -> ()
           | _ ->
               do {
-                match Adef.od_of_codate p.birth with
+                match Adef.od_of_codate (get_birth p) with
                 [ Some (Dgreg dt _) ->
                     if dt.prec = Sure && dt.day <> 0 && dt.month <> 0 &&
                        dt.month = mois && dt.delta = 0 then
@@ -80,7 +78,7 @@ value gen_print conf base mois f_scan dead_people =
                       else ()
                     else ()
                 | _ -> () ];
-                match p.death with
+                match get_death p with
                 [ Death dr d ->
                     match Adef.date_of_cdate d with
                     [ Dgreg dt _ ->
@@ -104,13 +102,14 @@ value gen_print conf base mois f_scan dead_people =
     Wserver.wprint "<ul>\n";
     for j = 1 to 31 do {
       if tab.(pred j) <> [] then do {
-        html_li conf;
-        Wserver.wprint "%d\n" j;
-        let liste =
-          Sort.list (fun (p1, a1, _, _) (p2, a2, _, _) -> a1 <= a2)
-            tab.(pred j)
-        in
-        print_anniversary_day conf base dead_people liste;
+        tag "li" begin
+          Wserver.wprint "%d\n" j;
+          let liste =
+            List.sort (fun (p1, a1, _, _) (p2, a2, _, _) -> compare a1 a2)
+              tab.(pred j)
+          in
+          print_anniversary_day conf base dead_people liste;
+        end
       }
       else ()
     };
@@ -121,12 +120,10 @@ value gen_print conf base mois f_scan dead_people =
 
 value print_anniversary_list conf base dead_people dt liste =
   let a_ref = dt.year in
-  do {
-    Wserver.wprint "<ul>\n";
+  tag "ul" begin
     List.iter
       (fun (p, a, date_event, txt_of) ->
-         do {
-           html_li conf;
+         stagn "li" begin
            if dead_people then do {
              Wserver.wprint "<em>";
              match date_event with
@@ -142,8 +139,8 @@ value print_anniversary_list conf base dead_people dt liste =
              Wserver.wprint ")</em>";
            }
            else do {
-             Wserver.wprint "\n%s" (txt_of conf base p);
-             match p.death with
+             Wserver.wprint "%s" (txt_of conf base p);
+             match get_death p with
              [ NotDead ->
                  do {
                    Wserver.wprint " <em>";
@@ -156,11 +153,9 @@ value print_anniversary_list conf base dead_people dt liste =
                  }
              | _ -> () ];
            };
-           Wserver.wprint "\n";
-         })
+         end)
       liste;
-    Wserver.wprint "</ul>\n";
-  }
+  end
 ;
 
 value f_scan conf base =
@@ -168,7 +163,7 @@ value f_scan conf base =
   fun () ->
     do {
       incr i;
-      if i.val < base.data.persons.len then
+      if i.val < nb_of_persons base then
         (pget conf base (Adef.iper_of_int i.val),
          referenced_person_title_text)
       else raise Not_found
@@ -181,52 +176,65 @@ value print_dead conf base mois =
   gen_print conf base mois (f_scan conf base) True
 ;
 
-value print_birth_day conf base day_name verb wd dt list =
-  do {
-    Wserver.wprint "\n";
-    html_p conf;
-    match list with
-    [ [] ->
+value print_birth_day conf base day_name fphrase wd dt list =
+  match list with
+  [ [] ->
+      tag "p" begin
         Wserver.wprint "%s %s.\n" (capitale (transl conf "no birthday"))
-          day_name
-    | _ ->
-        do {
-          Wserver.wprint "%s,\n" (capitale day_name);
-          Wserver.wprint "%s%s\n"
-            (std_color conf
-               ("<b>" ^
-                  transl_decline conf "on (weekday day month year)"
-                    (transl_nth conf "(week day)" wd ^ " " ^
-                       Date.code_dmy conf dt) ^
-                  "</b>"))
-            verb;
-          Wserver.wprint "%s\n"
-            (transl_a_of_b conf (transl conf "the birthday") "...");
-          print_anniversary_list conf base False dt list;
-        } ]
-  }
+          day_name;
+      end
+  | _ ->
+      do {
+        tag "p" begin
+          let txt =
+            transl_decline conf "on (weekday day month year)"
+              (transl_nth conf "(week day)" wd ^ " " ^ Date.code_dmy conf dt)
+          in
+          Wserver.wprint fphrase
+            (capitale day_name ^ ",\n" ^ std_color conf ("<b>" ^ txt ^ "</b>"))
+            (transl conf "the birthday");
+          Wserver.wprint "...\n";
+        end;
+        print_anniversary_list conf base False dt list;
+      } ]
 ;
 
 value propose_months conf mode =
-  tag "center" begin
-    tag "form" "method=get action=\"%s\"" conf.command begin
-      Util.hidden_env conf;
-      mode ();
-      tag "select" "name=v" begin
-        for i = 1 to 12 do {
-          Wserver.wprint "<option value=%d%s>%s\n" i
-            (if i = conf.today.month then " selected" else "")
-            (capitale (nominative (transl_nth conf "(month)" (i - 1))))
-        };
+  do {
+    begin_centered conf;
+    tag "table" "border=\"%d\"" conf.border begin
+      tag "tr" begin
+        tag "td" begin
+          tag "form" "method=\"get\" action=\"%s\"" conf.command begin
+            tag "p" begin
+              Util.hidden_env conf;
+              mode ();
+              tag "select" "name=\"v\"" begin
+                for i = 1 to 12 do {
+                  stagn "option" "value=\"%d\"%s" i
+                    (if i = conf.today.month then " selected=\"selected\""
+                     else "")
+                  begin
+                    Wserver.wprint "%s"
+                      (capitale
+                         (Util.translate_eval
+                            (transl_nth conf "(month)" (i - 1))));
+                  end
+                };
+              end;
+              xtag "input" "type=\"submit\" value=\"Ok\"";
+            end;
+          end;
+        end;
       end;
-      Wserver.wprint "<input type=submit value=\"Ok\">\n";
     end;
-  end
+    end_centered conf;
+  }
 ;
 
 value day_after d =
   let (day, r) =
-    if d.day >= nb_days_in_month d.month d.year then (1, 1)
+    if d.day >= CheckItem.nb_days_in_month d.month d.year then (1, 1)
     else (succ d.day, 0)
   in
   let (month, r) = if d.month + r > 12 then (1, 1) else (d.month + r, 0) in
@@ -234,27 +242,27 @@ value day_after d =
   {day = day; month = month; year = year; prec = Sure; delta = 0}
 ;
 
-value print_anniv conf base day_name verb wd dt list =
-  do {
-    Wserver.wprint "\n";
-    html_p conf;
-    match list with
-    [ [] ->
+value print_anniv conf base day_name fphrase wd dt list =
+  match list with
+  [ [] ->
+      tag "p" begin
         Wserver.wprint "%s %s.\n" (capitale (transl conf "no anniversary"))
-          day_name
-    | _ ->
-        do {
-          Wserver.wprint "%s, %s%s %s:\n" (capitale day_name)
-            (std_color conf
-               ("<b>" ^
-                  transl_decline conf "on (weekday day month year)"
-                    (transl_nth conf "(week day)" wd ^ " " ^
-                       Date.code_dmy conf dt) ^
-                  "</b>"))
-            verb (transl conf "the anniversary");
-          print_anniversary_list conf base True dt list;
-        } ]
-  }
+          day_name;
+      end
+  | _ ->
+      do {
+        tag "p" begin
+          let txt =
+            transl_decline conf "on (weekday day month year)"
+              (transl_nth conf "(week day)" wd ^ " " ^ Date.code_dmy conf dt)
+          in
+          Wserver.wprint fphrase
+            (capitale day_name ^ ",\n" ^ std_color conf ("<b>" ^ txt ^ "</b>"))
+            (transl conf "the anniversary");
+          Wserver.wprint "...\n";
+        end;
+        print_anniversary_list conf base True dt list;
+      } ]
 ;
 
 value print_marriage conf base month =
@@ -268,20 +276,19 @@ value print_marriage conf base month =
   do {
     header conf title;
     print_link_to_welcome conf True;
-    for i = 0 to base.data.families.len - 1 do {
-      let fam = base.data.families.get i in
+    for i = 0 to nb_of_families base - 1 do {
+      let fam = foi base (Adef.ifam_of_int i) in
       if is_deleted_family fam then ()
       else
-        match Adef.od_of_codate fam.marriage with
+        match Adef.od_of_codate (get_marriage fam) with
         [ Some (Dgreg {day = d; month = m; year = y; prec = Sure} _)
           when d <> 0 && m <> 0 ->
-            let cpl = base.data.couples.get i in
-            let father = pget conf base (father cpl) in
-            let mother = pget conf base (mother cpl) in
-            if m == month &&
+            let father = pget conf base (get_father fam) in
+            let mother = pget conf base (get_mother fam) in
+            if m = month &&
                authorized_age conf base father && not (is_hidden father) &&
                authorized_age conf base mother && not (is_hidden mother) then
-              tab.(pred d) := [(cpl, y) :: tab.(pred d)]
+              tab.(pred d) := [(fam, y) :: tab.(pred d)]
             else ()
         | _ -> () ]
     };
@@ -290,7 +297,7 @@ value print_marriage conf base month =
       match tab.(i - 1) with
       [ [] -> ()
       | l ->
-          let l = Sort.list (fun (fam1, y1) (fam2, y2) -> y1 < y2) l in
+          let l = List.sort (fun (fam1, y1) (fam2, y2) -> compare y1 y2) l in
           do {
             Wserver.wprint "\n";
             html_li conf;
@@ -298,15 +305,14 @@ value print_marriage conf base month =
             List.iter
               (fun (fam, year) ->
                  do {
-                   Wserver.wprint "\n";
                    html_li conf;
                    Wserver.wprint "%s"
                      (referenced_person_title_text conf base
-                        (pget conf base (father fam)));
+                        (pget conf base (get_father fam)));
                    Wserver.wprint "\n%s\n" (transl_nth conf "and" 0);
-                   Wserver.wprint "\n%s"
+                   Wserver.wprint "%s"
                      (referenced_person_title_text conf base
-                        (pget conf base (mother fam)));
+                        (pget conf base (get_mother fam)));
                    Wserver.wprint ", <em>%s %d</em>\n"
                      (transl conf "in (year)") year;
                  })
@@ -320,62 +326,56 @@ value print_marriage conf base month =
 ;
 
 value print_anniversaries_of_marriage conf base y list =
-  do {
-    Wserver.wprint "<ul>";
+  tag "ul" begin
     List.iter
       (fun (fam, year) ->
-         do {
-           Wserver.wprint "\n";
-           html_li conf;
-           Wserver.wprint "\n%s"
+         stagn "li" begin
+           Wserver.wprint "%s\n"
              (referenced_person_title_text conf base
-                (pget conf base (father fam)));
-           Wserver.wprint "\n%s\n" (transl_nth conf "and" 0);
-           Wserver.wprint "\n%s"
+                (pget conf base (get_father fam)));
+           Wserver.wprint "%s\n" (transl_nth conf "and" 0);
+           Wserver.wprint "%s"
              (referenced_person_title_text conf base
-                (pget conf base (mother fam)));
+                (pget conf base (get_mother fam)));
            Wserver.wprint ", <em>%s %d\n(" (transl conf "in (year)") year;
            Wserver.wprint (ftransl conf "%d years ago")
              (conf.today.year - year);
-           Wserver.wprint "</em>)\n";
-         })
+           Wserver.wprint "</em>)";
+         end)
       list;
-    Wserver.wprint "</ul>\n";
-  }
+  end
 ;
 
-value print_marriage_day conf base day_name verb wd dt list =
-  do {
-    Wserver.wprint "\n";
-    html_p conf;
-    match list with
-    [ [] ->
+value print_marriage_day conf base day_name fphrase wd dt list =
+  match list with
+  [ [] ->
+      tag "p" begin
         Wserver.wprint "%s %s.\n" (capitale (transl conf "no anniversary"))
-          day_name
-    | _ ->
-        do {
-          Wserver.wprint "%s,\n" (capitale day_name);
-          Wserver.wprint "%s%s\n"
-            (std_color conf
-               ("<b>" ^
-                  transl_decline conf "on (weekday day month year)"
-                    (transl_nth conf "(week day)" wd ^ " " ^
-                       Date.code_dmy conf dt) ^
-                  "</b>"))
-            verb;
-          Wserver.wprint "%s\n"
-            (transl_a_of_b conf
-               (transl conf "the anniversary of marriage") "...");
-          print_anniversaries_of_marriage conf base dt.year list;
-        } ]
-  }
+          day_name;
+      end
+  | _ ->
+      do {
+        tag "p" begin
+          Wserver.wprint fphrase
+            (capitale day_name ^ ",\n" ^
+             std_color conf
+                ("<b>" ^
+                   transl_decline conf "on (weekday day month year)"
+                     (transl_nth conf "(week day)" wd ^ " " ^
+                        Date.code_dmy conf dt) ^
+                   "</b>"))
+            (transl conf "the anniversary of marriage");
+          Wserver.wprint "...\n";
+        end;
+        print_anniversaries_of_marriage conf base dt.year list;
+      } ]
 ;
 
 value match_dates conf base p d1 d2 =
-  if d1.day == d2.day && d1.month == d2.month then authorized_age conf base p
+  if d1.day = d2.day && d1.month = d2.month then authorized_age conf base p
   else if
-    d1.day == 29 && d1.month == 2 && d2.day == 1 && d2.month = 3 &&
-    not (leap_year d2.year) then
+    d1.day = 29 && d1.month = 2 && d2.day = 1 && d2.month = 3 &&
+    not (CheckItem.leap_year d2.year) then
     authorized_age conf base p
   else False
 ;
@@ -393,7 +393,7 @@ value gen_print_menu_birth conf base f_scan mode =
     try
       while True do {
         let (p, txt_of) = f_scan () in
-        match (Adef.od_of_codate p.birth, p.death) with
+        match (Adef.od_of_codate (get_birth p), get_death p) with
         [ (Some (Dgreg d _), NotDead | DontKnowIfDead) ->
             if d.prec = Sure && d.day <> 0 && d.month <> 0 then
               if match_dates conf base p d conf.today then
@@ -411,18 +411,18 @@ value gen_print_menu_birth conf base f_scan mode =
     List.iter
       (fun xx ->
          xx.val :=
-           Sort.list (fun (p1, a1, _, _) (p2, a2, _, _) -> a1 <= a2) xx.val)
+           List.sort (fun (p1, a1, _, _) (p2, a2, _, _) -> compare a1 a2)
+             xx.val)
       [list_tod; list_tom; list_aft];
-    print_birth_day conf base (transl conf "today") (transl conf ", it is")
-      conf.today_wd conf.today list_tod.val;
+    print_birth_day conf base (transl conf "today")
+      (ftransl conf "%s, it is %s of") conf.today_wd conf.today list_tod.val;
     print_birth_day conf base (transl conf "tomorrow")
-      (transl conf ", it will be") ((conf.today_wd + 1) mod 7) tom
+      (ftransl conf "%s, it will be %s of") ((conf.today_wd + 1) mod 7) tom
       list_tom.val;
     print_birth_day conf base (transl conf "the day after tomorrow")
-      (transl conf ", it will be") ((conf.today_wd + 2) mod 7) aft
+      (ftransl conf "%s, it will be %s of") ((conf.today_wd + 2) mod 7) aft
       list_aft.val;
     Wserver.wprint "\n";
-    html_p conf;
     propose_months conf mode;
     Wserver.wprint "\n";
     trailer conf;
@@ -431,20 +431,21 @@ value gen_print_menu_birth conf base f_scan mode =
 
 value print_menu_birth conf base =
   let i = ref (-1) in
+  let nb_per = nb_of_persons base in
   let f_scan () =
     do {
       incr i;
-      if i.val < base.data.persons.len then
+      if i.val < nb_per then
         (pget conf base (Adef.iper_of_int i.val),
          referenced_person_title_text)
       else raise Not_found
     }
   in
-  let mode () = Wserver.wprint "<input type=hidden name=m value=AN>\n" in
+  let mode () = xtag "input" "type=\"hidden\" name=\"m\" value=\"AN\"" in
   gen_print_menu_birth conf base f_scan mode
 ;
 
-value print_menu_dead conf base =
+value gen_print_menu_dead conf base f_scan mode =
   let title _ =
     Wserver.wprint "%s"
       (capitale (transl conf "anniversaries of dead people"))
@@ -457,87 +458,95 @@ value print_menu_dead conf base =
   do {
     header conf title;
     print_link_to_welcome conf True;
-    for i = 0 to base.data.persons.len - 1 do {
-      let p = pget conf base (Adef.iper_of_int i) in
-      match p.death with
-      [ NotDead | DontKnowIfDead -> ()
-      | _ ->
-          do {
-            match Adef.od_of_codate p.birth with
-            [ Some (Dgreg d _) ->
-                if d.prec = Sure && d.day <> 0 && d.month <> 0 then
-                  if match_dates conf base p d conf.today then
-                    list_tod.val :=
-                      [(p, d.year, DeBirth, referenced_person_title_text) ::
-                       list_tod.val]
-                  else if match_dates conf base p d tom then
-                    list_tom.val :=
-                      [(p, d.year, DeBirth, referenced_person_title_text) ::
-                       list_tom.val]
-                  else if match_dates conf base p d aft then
-                    list_aft.val :=
-                      [(p, d.year, DeBirth, referenced_person_title_text) ::
-                       list_aft.val]
-                  else ()
-                else ()
-            | _ -> () ];
-            match p.death with
-            [ Death dr d ->
-                match Adef.date_of_cdate d with
-                [ Dgreg d _ ->
-                    if d.prec = Sure && d.day <> 0 && d.month <> 0 then
-                      if match_dates conf base p d conf.today then
-                        list_tod.val :=
-                          [(p, d.year, DeDeath dr,
-                            referenced_person_title_text) ::
-                           list_tod.val]
-                      else if match_dates conf base p d tom then
-                        list_tom.val :=
-                          [(p, d.year, DeDeath dr,
-                            referenced_person_title_text) ::
-                           list_tom.val]
-                      else if match_dates conf base p d aft then
-                        list_aft.val :=
-                          [(p, d.year, DeDeath dr,
-                            referenced_person_title_text) ::
-                           list_aft.val]
-                      else ()
+    try
+      while True do {
+        let (p, txt_of) = f_scan () in
+        match get_death p with
+        [ NotDead | DontKnowIfDead -> ()
+        | _ ->
+            do {
+              match Adef.od_of_codate (get_birth p) with
+              [ Some (Dgreg d _) ->
+                  if d.prec = Sure && d.day <> 0 && d.month <> 0 then
+                    if match_dates conf base p d conf.today then
+                      list_tod.val :=
+                        [(p, d.year, DeBirth, txt_of) :: list_tod.val]
+                    else if match_dates conf base p d tom then
+                      list_tom.val :=
+                        [(p, d.year, DeBirth, txt_of) :: list_tom.val]
+                    else if match_dates conf base p d aft then
+                      list_aft.val :=
+                        [(p, d.year, DeBirth, txt_of) :: list_aft.val]
                     else ()
-                | _ -> () ]
-            | _ -> () ];
-          } ]
-    };
+                  else ()
+              | _ -> () ];
+              match get_death p with
+              [ Death dr d ->
+                  match Adef.date_of_cdate d with
+                  [ Dgreg d _ ->
+                      if d.prec = Sure && d.day <> 0 && d.month <> 0 then
+                        if match_dates conf base p d conf.today then
+                          list_tod.val :=
+                            [(p, d.year, DeDeath dr, txt_of) :: list_tod.val]
+                        else if match_dates conf base p d tom then
+                          list_tom.val :=
+                            [(p, d.year, DeDeath dr, txt_of) :: list_tom.val]
+                        else if match_dates conf base p d aft then
+                          list_aft.val :=
+                            [(p, d.year, DeDeath dr, txt_of) :: list_aft.val]
+                        else ()
+                      else ()
+                  | _ -> () ]
+              | _ -> () ];
+            } ]
+      }
+    with
+    [ Not_found -> () ];
     List.iter
       (fun xx ->
          xx.val :=
-           Sort.list (fun (p1, a1, _, _) (p2, a2, _, _) -> a1 <= a2) xx.val)
+           List.sort (fun (p1, a1, _, _) (p2, a2, _, _) -> compare a1 a2)
+             xx.val)
       [list_tod; list_tom; list_aft];
-    print_anniv conf base (transl conf "today") (transl conf ", it is")
-      conf.today_wd conf.today list_tod.val;
+    print_anniv conf base (transl conf "today")
+      (ftransl conf "%s, it is %s of") conf.today_wd conf.today list_tod.val;
     print_anniv conf base (transl conf "tomorrow")
-      (transl conf ", it will be") ((conf.today_wd + 1) mod 7) tom
+      (ftransl conf "%s, it will be %s of") ((conf.today_wd + 1) mod 7) tom
       list_tom.val;
     print_anniv conf base (transl conf "the day after tomorrow")
-      (transl conf ", it will be") ((conf.today_wd + 2) mod 7) aft
+      (ftransl conf "%s, it will be %s of") ((conf.today_wd + 2) mod 7) aft
       list_aft.val;
     Wserver.wprint "\n";
-    html_p conf;
-    let mode () = Wserver.wprint "<input type=hidden name=m value=AD>\n" in
     propose_months conf mode;
     Wserver.wprint "\n";
     trailer conf;
   }
 ;
 
+value print_menu_dead conf base =
+  let i = ref (-1) in
+  let f_scan () =
+    do {
+      incr i;
+      if i.val < nb_of_persons base then
+        (pget conf base (Adef.iper_of_int i.val),
+         referenced_person_title_text)
+      else raise Not_found
+    }
+  in
+  let mode () = xtag "input" "type=\"hidden\" name=\"m\" value=\"AD\"" in
+  gen_print_menu_dead conf base f_scan mode
+;
+
 value match_mar_dates conf base cpl d1 d2 =
-  if d1.day == d2.day && d1.month == d2.month then
-    authorized_age conf base (pget conf base (father cpl)) &&
-    authorized_age conf base (pget conf base (mother cpl))
+  if d1.day = d2.day && d1.month = d2.month then
+    authorized_age conf base (pget conf base (get_father cpl)) &&
+    authorized_age conf base (pget conf base (get_mother cpl))
   else if
-    d1.day == 29 && d1.month == 2 && d2.day == 1 && d2.month = 3 &&
-    not (leap_year d2.year) then
-    authorized_age conf base (pget conf base (father cpl)) &&
-    authorized_age conf base (pget conf base (mother cpl))
+    d1.day = 29 && d1.month = 2 && d2.day = 1 && d2.month = 3 &&
+    not (CheckItem.leap_year d2.year) then
+    authorized_age conf base (pget conf base (get_father cpl)) &&
+    authorized_age conf base (pget conf base (get_mother cpl))
   else False
 ;
 
@@ -553,11 +562,11 @@ value print_menu_marriage conf base =
   do {
     header conf title;
     print_link_to_welcome conf True;
-    for i = 0 to base.data.families.len - 1 do {
-      let fam = base.data.families.get i in
+    for i = 0 to nb_of_families base - 1 do {
+      let fam = foi base (Adef.ifam_of_int i) in
       if is_deleted_family fam then ()
       else
-        match Adef.od_of_codate fam.marriage with
+        match Adef.od_of_codate (get_marriage fam) with
         [ Some (Dgreg d _) when d.day <> 0 && d.month <> 0 && d.prec = Sure ->
             let update_list cpl =
               if match_mar_dates conf base cpl d conf.today then
@@ -568,30 +577,29 @@ value print_menu_marriage conf base =
                 list_aft.val := [(cpl, d.year) :: list_aft.val]
               else ()
             in
-            let cpl = base.data.couples.get i in
             if conf.use_restrict then
-              let father = pget conf base (father cpl) in
-              let mother = pget conf base (mother cpl) in
+              let father = pget conf base (get_father fam) in
+              let mother = pget conf base (get_mother fam) in
               if not (is_hidden father) && not (is_hidden mother) then
-                update_list cpl
+                update_list fam
               else ()
-            else update_list cpl
+            else update_list fam
         | _ -> () ]
     };
     List.iter
-      (fun xx -> xx.val := Sort.list (fun (_, y1) (_, y2) -> y1 <= y2) xx.val)
+      (fun xx ->
+         xx.val := List.sort (fun (_, y1) (_, y2) -> compare y1 y2) xx.val)
       [list_tod; list_tom; list_aft];
-    print_marriage_day conf base (transl conf "today") (transl conf ", it is")
-      conf.today_wd conf.today list_tod.val;
+    print_marriage_day conf base (transl conf "today")
+      (ftransl conf "%s, it is %s of") conf.today_wd conf.today list_tod.val;
     print_marriage_day conf base (transl conf "tomorrow")
-      (transl conf ", it will be") ((conf.today_wd + 1) mod 7) tom
+      (ftransl conf "%s, it will be %s of") ((conf.today_wd + 1) mod 7) tom
       list_tom.val;
     print_marriage_day conf base (transl conf "the day after tomorrow")
-      (transl conf ", it will be") ((conf.today_wd + 2) mod 7) aft
+      (ftransl conf "%s, it will be %s of") ((conf.today_wd + 2) mod 7) aft
       list_aft.val;
     Wserver.wprint "\n";
-    html_p conf;
-    let mode () = Wserver.wprint "<input type=hidden name=m value=AM>\n" in
+    let mode () = xtag "input" "type=\"hidden\" name=\"m\" value=\"AM\"" in
     propose_months conf mode;
     Wserver.wprint "\n";
     trailer conf;
