@@ -1,15 +1,20 @@
 (* camlp4r ./pa_lock.cmo *)
-(* $Id: gwc.ml,v 4.26 2004/12/14 09:30:12 ddr Exp $ *)
-(* Copyright (c) 1998-2005 INRIA *)
+(* $Id: gwc.ml,v 5.53 2007/02/14 10:14:36 ddr Exp $ *)
+(* Copyright (c) 1998-2007 INRIA *)
 
+open Dbdisk;
 open Def;
-(*
-open Check;
-*)
-open Gutil;
 open Gwcomp;
+open Mutil;
 open Printf;
 
+type person = dsk_person;
+type ascend = dsk_ascend;
+type union = dsk_union;
+type family = dsk_family;
+type couple = dsk_couple;
+type descend = dsk_descend;
+
 type gen_min_person 'person 'string =
   { m_first_name : mutable 'string;
     m_surname : mutable 'string;
@@ -20,7 +25,7 @@ type gen_min_person 'person 'string =
     m_notes : mutable 'string }
 ;
 
-type min_person = gen_min_person iper istr;
+type min_person = gen_min_person iper dsk_istr;
 
 type cbase =
   { c_persons : mutable array min_person;
@@ -29,17 +34,19 @@ type cbase =
     c_couples : mutable array couple;
     c_descends : mutable array descend;
     c_strings : mutable array string;
-    c_bnotes : notes }
+    c_bnotes : mutable notes }
 ;
 
 type gen =
-  { g_strings : mutable Hashtbl.t string istr;
+  { g_strings : mutable Hashtbl.t string dsk_istr;
     g_names : mutable Hashtbl.t int iper;
     g_local_names : mutable Hashtbl.t (int * int) iper;
     g_pcnt : mutable int;
     g_fcnt : mutable int;
     g_scnt : mutable int;
     g_base : cbase;
+    g_wiznotes : mutable list (string * string);
+    g_patch_p : Hashtbl.t int person;
     g_def : mutable array bool;
     g_separate : mutable bool;
     g_shift : mutable int;
@@ -120,7 +127,7 @@ value unique_string gen x =
   try Hashtbl.find gen.g_strings x with
   [ Not_found ->
       do {
-        if gen.g_scnt == Array.length gen.g_base.c_strings then do {
+        if gen.g_scnt = Array.length gen.g_base.c_strings then do {
           let arr = gen.g_base.c_strings in
           let new_size = 2 * Array.length arr + 1 in
           let new_arr = Array.create new_size (no_string gen) in
@@ -137,8 +144,8 @@ value unique_string gen x =
 ;
 
 value no_family gen =
-  let empty = unique_string gen "" in
-  let cpl = couple False (Adef.iper_of_int 0) (Adef.iper_of_int 0)
+  let _ = unique_string gen "" in
+  let cpl = Adef.couple (Adef.iper_of_int 0) (Adef.iper_of_int 0)
   and des = {children = [| |]} in
   (cpl, des)
 ;
@@ -149,7 +156,7 @@ value make_person gen p n occ =
     {m_first_name = unique_string gen p; m_surname = unique_string gen n;
      m_occ = occ; m_rparents = []; m_related = []; m_sex = Neuter;
      m_notes = empty_string}
-  and a = no_ascend ()
+  and a = {parents = None; consang = Adef.fix (-1)}
   and u = {family = [| |]} in
   (p, a, u)
 ;
@@ -157,15 +164,15 @@ value make_person gen p n occ =
 value no_person gen = make_person gen "" "" 0;
 
 value new_iper gen =
-  if gen.g_pcnt == Array.length gen.g_base.c_persons then do {
+  if gen.g_pcnt = Array.length gen.g_base.c_persons then do {
     let per_arr = gen.g_base.c_persons in
     let asc_arr = gen.g_base.c_ascends in
     let uni_arr = gen.g_base.c_unions in
     let new_size = 2 * Array.length per_arr + 1 in
-    let (per_bidon, asc_bidon, uni_bidon) = no_person gen in
-    let new_per_arr = Array.create new_size per_bidon in
-    let new_asc_arr = Array.create new_size asc_bidon in
-    let new_uni_arr = Array.create new_size uni_bidon in
+    let (phony_per, phony_asc, phony_uni) = no_person gen in
+    let new_per_arr = Array.create new_size phony_per in
+    let new_asc_arr = Array.create new_size phony_asc in
+    let new_uni_arr = Array.create new_size phony_uni in
     let new_def = Array.create new_size False in
     Array.blit per_arr 0 new_per_arr 0 (Array.length per_arr);
     gen.g_base.c_persons := new_per_arr;
@@ -180,7 +187,7 @@ value new_iper gen =
 ;
 
 value new_ifam gen =
-  if gen.g_fcnt == Array.length gen.g_base.c_couples then do {
+  if gen.g_fcnt = Array.length gen.g_base.c_couples then do {
     let cpl_arr = gen.g_base.c_couples in
     let des_arr = gen.g_base.c_descends in
     let new_size = 2 * Array.length cpl_arr + 1 in
@@ -229,7 +236,7 @@ value find_person_by_global_name gen first_name surname occ =
     | [ip :: ipl] ->
         let p = poi gen.g_base ip in
         if Name.lower (p_first_name gen.g_base p) = first_name &&
-           Name.lower (p_surname gen.g_base p) = surname && p.m_occ == occ then
+           Name.lower (p_surname gen.g_base p) = surname && p.m_occ = occ then
           ip
         else loop ipl ]
   in
@@ -395,10 +402,7 @@ value insert_person gen so =
            | n -> "." ^ string_of_int n ])
           (p_surname gen.g_base x)
       else ();
-(*
-      x.birth := Adef.codate_None;
-      x.death := DontKnowIfDead;
-*)
+      flush stdout;
       check_error gen
     }
     else gen.g_def.(Adef.int_of_iper ip) := True;
@@ -429,7 +433,8 @@ value insert_person gen so =
          occ = 0; image = unique_string gen so.image;
          first_names_aliases =
            List.map (unique_string gen) so.first_names_aliases;
-         surnames_aliases = List.map (unique_string gen) so.surnames_aliases;
+         surnames_aliases =
+           List.map (unique_string gen) so.surnames_aliases;
          public_name = unique_string gen so.public_name;
          qualifiers = List.map (unique_string gen) so.qualifiers;
          aliases = List.map (unique_string gen) so.aliases;
@@ -450,7 +455,7 @@ value insert_person gen so =
          psources =
            unique_string gen
              (if so.psources = "" then default_source.val else so.psources);
-         cle_index = ip}
+         key_index = ip}
       in
       seek_out gen.g_per_index (Iovalue.sizeof_long * Adef.int_of_iper ip);
       output_binary_int gen.g_per_index (pos_out gen.g_per);
@@ -468,13 +473,13 @@ value insert_somebody gen =
   | Defined so -> insert_person gen so ]
 ;
 
-value verif_parents_non_deja_definis gen ix pere mere =
+value check_parents_not_already_defined gen ix fath moth =
   let x = poi gen.g_base ix in
-  match parents (aoi gen.g_base ix) with
+  match (aoi gen.g_base ix).parents with
   [ Some ifam ->
       let cpl = coi gen.g_base ifam in
-      let p = (father cpl) in
-      let m = (mother cpl) in
+      let p = Adef.father cpl in
+      let m = Adef.mother cpl in
       do {
         printf "
 I cannot add \"%s\", child of
@@ -483,8 +488,8 @@ I cannot add \"%s\", child of
 because this persons still exists as child of
     - \"%s\"
     - \"%s\".
-" (designation gen.g_base x) (designation gen.g_base pere)
-          (designation gen.g_base mere)
+" (designation gen.g_base x) (designation gen.g_base fath)
+          (designation gen.g_base moth)
           (designation gen.g_base (poi gen.g_base p))
           (designation gen.g_base (poi gen.g_base m));
         flush stdout;
@@ -498,8 +503,8 @@ because this persons still exists as child of
 ;
 
 value notice_sex gen p s =
-  if p.m_sex == Neuter then p.m_sex := s
-  else if p.m_sex == s || s == Neuter then ()
+  if p.m_sex = Neuter then p.m_sex := s
+  else if p.m_sex = s || s = Neuter then ()
   else do {
     printf "\nInconcistency about the sex of\n  %s %s\n"
       (p_first_name gen.g_base p) (p_surname gen.g_base p);
@@ -508,16 +513,16 @@ value notice_sex gen p s =
 ;
 
 value insert_family gen co fath_sex moth_sex witl fo deo =
-  let (pere, ipere) = insert_somebody gen (father co) in
-  let (mere, imere) = insert_somebody gen (mother co) in
+  let (fath, ifath) = insert_somebody gen (Adef.father co) in
+  let (moth, imoth) = insert_somebody gen (Adef.mother co) in
   let witl =
     List.map
       (fun (wit, sex) ->
          let (p, ip) = insert_somebody gen wit in
          do {
            notice_sex gen p sex;
-           if not (List.mem ipere p.m_related) then
-             p.m_related := [ipere :: p.m_related]
+           if not (List.mem ifath p.m_related) then
+             p.m_related := [ifath :: p.m_related]
            else ();
            ip
          })
@@ -525,9 +530,9 @@ value insert_family gen co fath_sex moth_sex witl fo deo =
   in
   let children =
     Array.map
-      (fun cle ->
-         let (e, ie) = insert_person gen cle in
-         do { notice_sex gen e cle.sex; ie })
+      (fun key ->
+         let (e, ie) = insert_person gen key in
+         do { notice_sex gen e key.sex; ie })
       deo.children
   in
   let comment = unique_string gen fo.comment in
@@ -546,26 +551,33 @@ value insert_family gen co fath_sex moth_sex witl fo deo =
        divorce = fo.divorce; comment = comment;
        origin_file = unique_string gen fo.origin_file; fsources = fsources;
        fam_index = Adef.ifam_of_int i}
-    and cpl = couple False ipere imere
+    and cpl = Adef.couple ifath imoth
     and des = {children = children} in
-    let fath_uni = uoi gen.g_base ipere in
-    let moth_uni = uoi gen.g_base imere in
+    let fath_uni = uoi gen.g_base ifath in
+    let moth_uni = uoi gen.g_base imoth in
     seek_out gen.g_fam_index (Iovalue.sizeof_long * i);
     output_binary_int gen.g_fam_index (pos_out gen.g_fam);
     output_item_value gen.g_fam (fam : family);
     gen.g_base.c_couples.(gen.g_fcnt) := cpl;
     gen.g_base.c_descends.(gen.g_fcnt) := des;
     gen.g_fcnt := gen.g_fcnt + 1;
-    fath_uni.family := Array.append fath_uni.family [| Adef.ifam_of_int i |];
-    moth_uni.family := Array.append moth_uni.family [| Adef.ifam_of_int i |];
-    notice_sex gen pere fath_sex;
-    notice_sex gen mere moth_sex;
+    let fath_uni =
+      {family = Array.append fath_uni.family [| Adef.ifam_of_int i |]}
+    in
+    gen.g_base.c_unions.(Adef.int_of_iper ifath) := fath_uni;
+    let moth_uni =
+      {family = Array.append moth_uni.family [| Adef.ifam_of_int i |]}
+    in
+    gen.g_base.c_unions.(Adef.int_of_iper imoth) := moth_uni;
+    notice_sex gen fath fath_sex;
+    notice_sex gen moth moth_sex;
     Array.iter
       (fun ix ->
-         let a = aoi gen.g_base ix in
+         let a = gen.g_base.c_ascends.(Adef.int_of_iper ix) in
          do {
-           verif_parents_non_deja_definis gen ix pere mere;
-           set_parents a (Some (Adef.ifam_of_int i));
+           check_parents_not_already_defined gen ix fath moth;
+           let a = {(a) with parents = Some (Adef.ifam_of_int i)} in
+           gen.g_base.c_ascends.(Adef.int_of_iper ix) := a
          })
       children;
   }
@@ -584,7 +596,7 @@ value insert_notes fname gen key str =
       if sou gen.g_base p.m_notes <> "" then do {
         printf "\nFile \"%s\"\n" fname;
         printf "Notes already defined for \"%s%s %s\"\n"
-          key.pk_first_name (if occ == 0 then "" else "." ^ string_of_int occ)
+          key.pk_first_name (if occ = 0 then "" else "." ^ string_of_int occ)
           key.pk_surname;
         check_error gen
       }
@@ -593,17 +605,35 @@ value insert_notes fname gen key str =
       do {
         printf "File \"%s\"\n" fname;
         printf "*** warning: undefined person: \"%s%s %s\"\n"
-          key.pk_first_name (if occ == 0 then "" else "." ^ string_of_int occ)
+          key.pk_first_name (if occ = 0 then "" else "." ^ string_of_int occ)
           key.pk_surname;
         flush stdout;
       } ]
 ;
 
-value insert_bnotes fname gen str =
-  do {
-    gen.g_base.c_bnotes.nread := fun _ -> str;
-    gen.g_base.c_bnotes.norigin_file := fname;
-  }
+value insert_bnotes fname gen nfname str = do {
+  let old_nread = gen.g_base.c_bnotes.nread in
+  let nfname =
+    if nfname = "" then ""
+    else
+      match NotesLinks.check_file_name nfname with
+      [ Some (dl, f) -> List.fold_right Filename.concat dl f
+      | None -> "bad" ]
+  in
+  let bnotes =
+    {nread f n = if f = nfname then str else old_nread f n;
+     norigin_file = fname;
+     efiles =
+       if nfname <> "" then
+         let efiles = gen.g_base.c_bnotes.efiles () in
+         fun () -> [nfname :: efiles]
+       else gen.g_base.c_bnotes.efiles}
+  in
+  gen.g_base.c_bnotes := bnotes;
+};
+
+value insert_wiznote fname gen wizid str =
+  gen.g_wiznotes := [(wizid, str) :: gen.g_wiznotes]
 ;
 
 value map_option f =
@@ -625,8 +655,8 @@ value insert_relation_parent gen ip s k =
 
 value insert_relation gen ip r =
   {r_type = r.r_type;
-   r_fath = map_option (insert_relation_parent gen ip Neuter) r.r_fath;
-   r_moth = map_option (insert_relation_parent gen ip Neuter) r.r_moth;
+   r_fath = map_option (insert_relation_parent gen ip Male) r.r_fath;
+   r_moth = map_option (insert_relation_parent gen ip Female) r.r_moth;
    r_sources = unique_string gen r.r_sources}
 ;
 
@@ -636,7 +666,7 @@ value insert_relations fname gen sb sex rl =
     printf "\nFile \"%s\"\n" fname;
     printf "Relations already defined for \"%s%s %s\"\n"
       (sou gen.g_base p.m_first_name)
-      (if p.m_occ == 0 then "" else "." ^ string_of_int p.m_occ)
+      (if p.m_occ = 0 then "" else "." ^ string_of_int p.m_occ)
       (sou gen.g_base p.m_surname);
     check_error gen
   }
@@ -652,15 +682,14 @@ value insert_syntax fname gen =
   [ Family cpl fs ms witl fam des -> insert_family gen cpl fs ms witl fam des
   | Notes key str -> insert_notes fname gen key str
   | Relations sb sex rl -> insert_relations fname gen sb sex rl
-  | Bnotes str -> insert_bnotes fname gen str ]
+  | Bnotes nfname str -> insert_bnotes fname gen nfname str
+  | Wnotes wizid str -> insert_wiznote fname gen wizid str ]
 ;
 
-value insert_comp_families gen (x, separate, shift) =
-(**)
-  let _ = do { Printf.eprintf "%s " x; flush stderr; } in
-(**)
-  let ic = open_in_bin x in
+value insert_comp_families gen run (x, separate, shift) =
   do {
+    run ();
+    let ic = open_in_bin x in
     check_magic x ic;
     gen.g_separate := separate;
     gen.g_shift := shift;
@@ -682,61 +711,68 @@ value force = ref False;
 value do_consang = ref False;
 value pr_stats = ref False;
 
-value cache_of tab =
-  let c =
-    {array = fun _ -> tab; get = fun []; len = Array.length tab;
-     clear_array = fun _ -> ()}
-  in
-  do { c.get := fun i -> (c.array ()).(i); c }
+value record_access_of tab =
+  {load_array () = (); get i = tab.(i); set i v = tab.(i) := v;
+   output_array oc = output_value_no_sharing oc (tab : array _);
+   len = Array.length tab; clear_array () = ()}
 ;
 
 value no_istr_iper_index = {find = fun []; cursor = fun []; next = fun []};
 
-value persons_cache per_index_ic per_ic persons =
-  let get_fun i =
+value persons_record_access gen per_index_ic per_ic persons =
+  let read_person_in_temp_file i =
     let mp = persons.(i) in
-    do {
-      seek_in per_index_ic (Iovalue.sizeof_long * i);
-      let pos = input_binary_int per_index_ic in
-      seek_in per_ic pos;
-      let p : person =
-        match input_char per_ic with
-        [ 'D' -> input_item_value per_ic
-        | 'U' ->
-            let empty_string = Adef.istr_of_int 0 in
-            {first_name = empty_string; surname = empty_string;
-             occ = 0; image = empty_string; first_names_aliases = [];
-             surnames_aliases = []; public_name = empty_string;
-             qualifiers = [];
-             aliases = []; titles = []; rparents = []; related = [];
-             occupation = empty_string; sex = Neuter; access = IfTitles;
-             birth = Adef.codate_None; birth_place = empty_string;
-             birth_src = empty_string; baptism = Adef.codate_None;
-             baptism_place = empty_string; baptism_src = empty_string;
-             death = DontKnowIfDead; death_place = empty_string;
-             death_src = empty_string; burial = UnknownBurial;
-             burial_place = empty_string; burial_src = empty_string;
-             notes = empty_string; psources = empty_string;
-             cle_index = Adef.iper_of_int 0}
-        | _ -> assert False ]
+    let p =
+      let c =
+        try
+          do {
+            seek_in per_index_ic (Iovalue.sizeof_long * i);
+            let pos = input_binary_int per_index_ic in
+            seek_in per_ic pos;
+            input_char per_ic
+          }
+        with
+        [ End_of_file -> 'U' ]
       in
-      p.first_name := mp.m_first_name;
-      p.surname := mp.m_surname;
-      p.occ := mp.m_occ;
-      p.rparents := mp.m_rparents;
-      p.related := mp.m_related;
-      p.sex := mp.m_sex;
-      p.notes := mp.m_notes;
-      p.cle_index := Adef.iper_of_int i;
-      p
-    }
+      match c with
+      [ 'D' -> (input_item_value per_ic : person)
+      | 'U' ->
+          let empty_string = Adef.istr_of_int 0 in
+          {first_name = empty_string; surname = empty_string;
+           occ = 0; image = empty_string; first_names_aliases = [];
+           surnames_aliases = []; public_name = empty_string;
+           qualifiers = [];
+           aliases = []; titles = []; rparents = []; related = [];
+           occupation = empty_string; sex = Neuter; access = IfTitles;
+           birth = Adef.codate_None; birth_place = empty_string;
+           birth_src = empty_string; baptism = Adef.codate_None;
+           baptism_place = empty_string; baptism_src = empty_string;
+           death = DontKnowIfDead; death_place = empty_string;
+           death_src = empty_string; burial = UnknownBurial;
+           burial_place = empty_string; burial_src = empty_string;
+           notes = empty_string; psources = empty_string;
+           key_index = Adef.iper_of_int 0}
+      | _ -> assert False ]
+    in
+    {(p) with
+     first_name = mp.m_first_name; surname = mp.m_surname;
+     occ = mp.m_occ; rparents = mp.m_rparents; related = mp.m_related;
+     sex = mp.m_sex; notes = mp.m_notes; key_index = Adef.iper_of_int i}
   in
-  {array = fun _ -> failwith "bug: accessing persons array";
-   get = get_fun; len = Array.length persons;
-   clear_array = fun _ -> ()}
+  let get_fun i =
+    try Hashtbl.find gen.g_patch_p i with
+    [ Not_found -> read_person_in_temp_file i ]
+  in
+  let len = Array.length persons in
+  {load_array () = (); get = get_fun;
+   set i v = failwith "bug: setting persons array";
+   output_array oc = Mutil.output_array_no_sharing oc get_fun len;
+   len = len; clear_array () = ()}
 ;
 
-value families_cache fam_index_ic fam_ic len =
+value part_file = ref "";
+
+value families_record_access fam_index_ic fam_ic len =
   let get_fun i =
     do {
       seek_in fam_index_ic (Iovalue.sizeof_long * i);
@@ -746,17 +782,28 @@ value families_cache fam_index_ic fam_ic len =
       fam
     }
   in
-  {array = fun _ -> failwith "bug: accessing family array";
-   get = get_fun; len = len; clear_array = fun _ -> ()}
+  {load_array () = (); get = get_fun;
+   set i v = failwith "bug: setting family array";
+   output_array oc = output_array_no_sharing oc get_fun len;
+   len = len; clear_array () = ()}
+;
+
+value input_particles part_file =
+  if part_file = "" then
+    ["af "; "d'"; "dal "; "de "; "di "; "du "; "of "; "van ";
+     "von und zu "; "von "; "zu "; "zur ";
+     "AF "; "D'"; "DAL "; "DE "; "DI "; "DU "; "OF "; "VAN ";
+     "VON UND ZU "; "VON "; "ZU "; "ZUR "]
+  else Mutil.input_particles part_file
 ;
 
 value empty_base : cbase =
   {c_persons = [| |]; c_ascends = [| |]; c_unions = [| |];
    c_couples = [| |]; c_descends = [| |]; c_strings = [| |];
-   c_bnotes = {nread = fun _ -> ""; norigin_file = ""}}
+   c_bnotes = {nread = fun _ _ -> ""; norigin_file = ""; efiles _ = []}}
 ;
 
-value linked_base gen per_index_ic per_ic fam_index_ic fam_ic : Def.base =
+value linked_base gen per_index_ic per_ic fam_index_ic fam_ic bdir =
 (**)
   let _ =
     do {
@@ -808,87 +855,167 @@ value linked_base gen per_index_ic per_ic fam_index_ic fam_ic : Def.base =
     let a = Array.sub gen.g_base.c_strings 0 gen.g_scnt in
     do { gen.g_base.c_strings := [| |]; a }
   in
+  let particles = input_particles part_file.val in
   let bnotes = gen.g_base.c_bnotes in
   let base_data =
-    {persons = persons_cache per_index_ic per_ic persons;
-     ascends = cache_of ascends;
-     unions = cache_of unions;
-     families = families_cache fam_index_ic fam_ic gen.g_fcnt;
+    {persons = persons_record_access gen per_index_ic per_ic persons;
+     ascends = record_access_of ascends;
+     unions = record_access_of unions;
+     families = families_record_access fam_index_ic fam_ic gen.g_fcnt;
      visible = { v_write = fun []; v_get = fun [] };
-     couples = cache_of couples; descends = cache_of descends;
-     strings = cache_of strings; bnotes = bnotes}
+     couples = record_access_of couples; descends = record_access_of descends;
+     strings = record_access_of strings; particles = particles;
+     bnotes = bnotes; bdir = bdir}
   in
   let base_func =
-    {persons_of_name = fun []; strings_of_fsname = fun [];
-     index_of_string = fun []; persons_of_surname = no_istr_iper_index;
+    {person_of_key = fun [];
+     persons_of_name = fun []; strings_of_fsname = fun [];
+     persons_of_surname = no_istr_iper_index;
      persons_of_first_name = no_istr_iper_index;
      patch_person = fun []; patch_ascend = fun []; patch_union = fun [];
      patch_family = fun []; patch_couple = fun []; patch_descend = fun [];
-     patch_string = fun []; patch_name = fun []; commit_patches = fun [];
-     commit_notes = fun []; patched_ascends = fun []; cleanup = fun () -> ()}
+     insert_string = fun []; patch_name = fun []; commit_patches = fun [];
+     commit_notes = fun []; patched_ascends = fun [];
+     is_patched_person _ = False; cleanup = fun () -> ()}
   in
   {data = base_data; func = base_func}
 ;
 
-value link gwo_list =
-  let gen =
-    {g_strings = Hashtbl.create 20011; g_names = Hashtbl.create 20011;
-     g_local_names = Hashtbl.create 20011; g_pcnt = 0; g_fcnt = 0;
-     g_scnt = 0; g_base = empty_base; g_def = [| |]; g_separate = False;
-     g_shift = 0; g_errored = False;
-     g_per_index = open_out_bin "gwc_per_index";
-     g_per = open_out_bin "gwc_per";
-     g_fam_index = open_out_bin "gwc_fam_index";
-     g_fam = open_out_bin "gwc_fam" }
-  in
-  let per_index_ic = open_in_bin "gwc_per_index" in
-  let per_ic = open_in_bin "gwc_per" in
-  let fam_index_ic = open_in_bin "gwc_fam_index" in
-  let fam_ic = open_in_bin "gwc_fam" in
-  let istr_empty = unique_string gen "" in
-  let istr_quest = unique_string gen "?" in
+value fold_option fsome vnone =
+  fun
+  [ Some v -> fsome v
+  | None -> vnone ]
+;
+
+value link gwo_list tmp_dir bdir =
+  let tmp_per_index = Filename.concat tmp_dir "gwc_per_index" in
+  let tmp_per = Filename.concat tmp_dir "gwc_per" in
+  let tmp_fam_index = Filename.concat tmp_dir "gwc_fam_index" in
+  let tmp_fam = Filename.concat tmp_dir "gwc_fam" in
   do {
+    let gen =
+      {g_strings = Hashtbl.create 20011; g_names = Hashtbl.create 20011;
+       g_local_names = Hashtbl.create 20011; g_pcnt = 0; g_fcnt = 0;
+       g_scnt = 0; g_base = empty_base; g_patch_p = Hashtbl.create 20011;
+       g_wiznotes = [];
+       g_def = [| |]; g_separate = False; g_shift = 0; g_errored = False;
+       g_per_index = open_out_bin tmp_per_index;
+       g_per = open_out_bin tmp_per;
+       g_fam_index = open_out_bin tmp_fam_index;
+       g_fam = open_out_bin tmp_fam }
+    in
+    let per_index_ic = open_in_bin tmp_per_index in
+    let per_ic = open_in_bin tmp_per in
+    let fam_index_ic = open_in_bin tmp_fam_index in
+    let fam_ic = open_in_bin tmp_fam in
+    let istr_empty = unique_string gen "" in
+    let istr_quest = unique_string gen "?" in
     assert (istr_empty = Adef.istr_of_int 0);
     assert (istr_quest = Adef.istr_of_int 1);
-    ifdef UNIX then Sys.remove "gwc_per_index" else ();
-    ifdef UNIX then Sys.remove "gwc_per" else ();
-    ifdef UNIX then Sys.remove "gwc_fam_index" else ();
-    ifdef UNIX then Sys.remove "gwc_fam" else ();
-    List.iter (insert_comp_families gen) gwo_list;
-    Printf.eprintf "\n"; flush stderr;
+    IFDEF UNIX THEN Sys.remove tmp_per_index ELSE () END;
+    IFDEF UNIX THEN Sys.remove tmp_per ELSE () END;
+    IFDEF UNIX THEN Sys.remove tmp_fam_index ELSE () END;
+    IFDEF UNIX THEN Sys.remove tmp_fam ELSE () END;
+    let ngwo = List.length gwo_list in
+    let run =
+      if ngwo < 10 then fun () -> ()
+      else if ngwo < 60 then
+        fun () -> do { Printf.eprintf "."; flush stderr; }
+      else do {
+        let bar_cnt = ref 0 in
+        let run () = do { ProgrBar.run bar_cnt.val ngwo; incr bar_cnt } in
+        ProgrBar.empty.val := 'o';
+        ProgrBar.full.val := '*';
+        ProgrBar.start ();
+        run
+      }
+    in
+    List.iter (insert_comp_families gen run) gwo_list;
+    if ngwo < 10 then ()
+    else if ngwo < 60 then do { Printf.eprintf "\n"; flush stderr }
+    else ProgrBar.finish ();
     close_out gen.g_per_index;
     close_out gen.g_per;
     close_out gen.g_fam_index;
     close_out gen.g_fam;
-    let base = linked_base gen per_index_ic per_ic fam_index_ic fam_ic in
+    Hashtbl.clear gen.g_strings;
+    Hashtbl.clear gen.g_names;
+    Hashtbl.clear gen.g_local_names;
+    Gc.compact ();
+    let dsk_base =
+      linked_base gen per_index_ic per_ic fam_index_ic fam_ic bdir
+    in
+    Hashtbl.clear gen.g_patch_p;
+    let base = Gwdb.base_of_base1 dsk_base in
     if do_check.val && gen.g_pcnt > 0 then do {
+      let changed_p (ip, p, o_sex, o_rpar) =
+        let p = Gwdb.dsk_person_of_person p in
+        let p =
+          {(p) with
+           sex = fold_option (fun s -> s) p.sex o_sex;
+           rparents =
+             fold_option
+               (List.map
+                  (Futil.map_relation_ps (fun p -> p)
+                     (fun s -> Adef.istr_of_int 0)))
+               p.rparents o_rpar}
+        in
+        let i = Adef.int_of_iper ip in
+        Hashtbl.replace gen.g_patch_p i p
+      in
       Check.check_base base (set_error base gen) (set_warning base)
-        (fun i -> gen.g_def.(i)) pr_stats.val;
+        (fun i -> gen.g_def.(i)) changed_p pr_stats.val;
       flush stdout;
     }
     else ();
-    if not gen.g_errored then
-      if do_consang.val then ConsangAll.compute base True False else ()
-    else exit 2;
-    base
+    if not gen.g_errored then do {
+      if do_consang.val then
+        let _ : option _ = ConsangAll.compute base True False in ()
+        else ();
+      Some (dsk_base, gen.g_wiznotes)
+    }
+    else None;
   }
 ;
 
-value output_command_line bname =
-  let bdir =
-    if Filename.check_suffix bname ".gwb" then bname else bname ^ ".gwb"
-  in
-  let oc = open_out (Filename.concat bdir "command.txt") in
+value write_file_contents fname text =
+  let oc = open_out fname in
   do {
-    fprintf oc "%s" Sys.argv.(0);
-    for i = 1 to Array.length Sys.argv - 1 do {
-      fprintf oc " %s" Sys.argv.(i)
-    };
-    fprintf oc "\n";
+    output_string oc text;
     close_out oc;
   }
 ;
 
+value output_wizard_notes bdir wiznotes = do {
+  let wizdir = Filename.concat bdir "wiznotes" in
+  Mutil.remove_dir wizdir;
+  if wiznotes = [] then ()
+  else do {
+    try Unix.mkdir wizdir 0o755 with _ -> ();
+    List.iter
+      (fun (wizid, text) ->
+         let fname = Filename.concat wizdir wizid ^ ".txt" in
+         write_file_contents fname text)
+      wiznotes;
+  }
+};
+
+value output_command_line bdir = do {
+  let oc = open_out (Filename.concat bdir "command.txt") in
+  fprintf oc "%s" Sys.argv.(0);
+  for i = 1 to Array.length Sys.argv - 1 do {
+    fprintf oc " %s" Sys.argv.(i)
+  };
+  fprintf oc "\n";
+  close_out oc;
+};
+
+value output_particles_file bdir particles = do {
+  let oc = open_out (Filename.concat bdir "particles.txt") in
+  List.iter (fun s -> fprintf oc "%s\n" (Mutil.tr ' ' '_' s)) particles;
+  close_out oc;
+};
+
 value separate = ref False;
 value shift = ref 0;
 value files = ref [];
@@ -905,9 +1032,12 @@ value speclist =
    ("-sh", Arg.Int (fun x -> shift.val := x),
     "<int> Shift all persons numbers in next files");
    ("-ds", Arg.String (fun s -> default_source.val := s), "\
-     set the source field for persons and families without source data");
-   ("-mem", Arg.Set Iobase.save_mem, " Save memory, but slower");
-   ("-nolock", Arg.Set Lock.no_lock_flag, " do not lock database.")]
+     <str> Set the source field for persons and families without source data");
+   ("-part", Arg.String (fun s -> part_file.val := s), "\
+     <file> Particles file (default = predefined particles)");
+   ("-mem", Arg.Set Outbase.save_mem, " Save memory, but slower");
+   ("-nolock", Arg.Set Lock.no_lock_flag, " do not lock database.");
+   ("-nofail", Arg.Set Gwcomp.no_fail, " no failure in case of error.")]
 ;
 
 value anonfun x =
@@ -961,18 +1091,38 @@ The database \"%s\" already exists. Use option -f to overwrite it.
         exit 2
       }
       else ();
-      lock (Iobase.lock_file out_file.val) with
-      [ Accept ->
-          let base = link (List.rev gwo.val) in
-          do { Gc.compact (); Iobase.output out_file.val base; }
+      lock (Mutil.lock_file out_file.val) with
+      [ Accept -> do {
+          let bdir =
+            let bname = out_file.val in
+            if Filename.check_suffix bname ".gwb" then bname
+            else bname ^ ".gwb"
+          in
+          let tmp_dir = Filename.concat "gw_tmp" bdir in
+          try Mutil.mkdir_p tmp_dir with _ -> ();
+          match link (List.rev gwo.val) tmp_dir bdir with
+          [ Some (dsk_base, wiznotes) ->
+              do {
+                Gc.compact ();
+                Outbase.output bdir dsk_base;
+                output_wizard_notes bdir wiznotes;
+                output_command_line bdir;
+                output_particles_file bdir dsk_base.data.particles;
+                try Mutil.remove_dir tmp_dir with _ -> ();
+                try Unix.rmdir "gw_tmp" with _ -> ();
+              }
+          | None -> do {
+              printf "Creation of database failed.\n";
+              flush stdout;
+              exit 2;
+            } ]
+        }
       | Refuse ->
           do {
             printf "Base is locked: cannot write it\n";
             flush stdout;
             exit 2
           } ];
-      output_command_line out_file.val;
-      ()
     }
     else ();
   }
