@@ -1,10 +1,11 @@
 (* camlp4r ./pa_html.cmo *)
-(* $Id: relationLink.ml,v 4.12 2004/12/14 09:30:17 ddr Exp $ *)
-(* Copyright (c) 1998-2005 INRIA *)
+(* $Id: relationLink.ml,v 5.18 2007/01/19 01:53:17 ddr Exp $ *)
+(* Copyright (c) 1998-2007 INRIA *)
 
 open Config;
 open Def;
-open Gutil;
+open Gwdb;
+open Hutil;
 open Util;
 
 (* Algorithm *)
@@ -46,24 +47,23 @@ value make_dist_tab conf base ia maxlev =
     (* optimization to be used 1/ if database not too big or 2/ running
     on machines with much memory *)
 (*
-    let _ = base.data.unions.array () in
-    let _ = base.data.descends.array () in
+    let () = load_unions_array base in
+    let () = load_descends_array base in
 *)
-    (**)
     let tstab = Util.create_topological_sort conf base in
     let module Pq =
       Pqueue.Make
         (struct type t = int; value leq x y = not (tsort_leq tstab x y); end)
     in
     let default = {dmin = infinity; dmax = 0; mark = False} in
-    let dist = Array.create base.data.persons.len default in
+    let dist = Array.create (nb_of_persons base) default in
     let q = ref Pq.empty in
     let add_children ip =
       let u = uget conf base ip in
-      for i = 0 to Array.length u.family - 1 do {
-        let des = doi base u.family.(i) in
-        for j = 0 to Array.length des.children - 1 do {
-          let k = Adef.int_of_iper des.children.(j) in
+      for i = 0 to Array.length (get_family u) - 1 do {
+        let des = foi base (get_family u).(i) in
+        for j = 0 to Array.length (get_children des) - 1 do {
+          let k = Adef.int_of_iper (get_children des).(j) in
           let d = dist.(k) in
           if not d.mark then do {
             dist.(k) := {dmin = infinity; dmax = 0; mark = True};
@@ -79,11 +79,11 @@ value make_dist_tab conf base ia maxlev =
       while not (Pq.is_empty q.val) do {
         let (k, nq) = Pq.take q.val in
         q.val := nq;
-        match parents (aget conf base (Adef.iper_of_int k)) with
+        match get_parents (aget conf base (Adef.iper_of_int k)) with
         [ Some ifam ->
-            let cpl = coi base ifam in
-            let dfath = dist.(Adef.int_of_iper (father cpl)) in
-            let dmoth = dist.(Adef.int_of_iper (mother cpl)) in
+            let cpl = foi base ifam in
+            let dfath = dist.(Adef.int_of_iper (get_father cpl)) in
+            let dmoth = dist.(Adef.int_of_iper (get_mother cpl)) in
             do {
               dist.(k).dmin := min dfath.dmin dmoth.dmin + 1;
               dist.(k).dmax := max dfath.dmax dmoth.dmax + 1;
@@ -99,16 +99,16 @@ value make_dist_tab conf base ia maxlev =
 
 value find_first_branch conf base (dmin, dmax) ia =
   find [] where rec find br len ip sp =
-    if ip == ia then if len == 0 then Some br else None
-    else if len == 0 then None
+    if ip = ia then if len = 0 then Some br else None
+    else if len = 0 then None
     else if len < dmin ip || len > dmax ip then None
     else
-      match parents (aget conf base ip) with
+      match get_parents (aget conf base ip) with
       [ Some ifam ->
-          let cpl = coi base ifam in
-          match find [(ip, sp) :: br] (len - 1) (father cpl) Male with
+          let cpl = foi base ifam in
+          match find [(ip, sp) :: br] (len - 1) (get_father cpl) Male with
           [ Some _ as r -> r
-          | None -> find [(ip, sp) :: br] (len - 1) (mother cpl) Female ]
+          | None -> find [(ip, sp) :: br] (len - 1) (get_mother cpl) Female ]
       | None -> None ]
 ;
 
@@ -121,29 +121,29 @@ value rec next_branch_same_len conf base dist backward missing ia sa ipl =
         [ Female ->
             next_branch_same_len conf base dist True (missing + 1) ip sp ipl1
         | Male ->
-            match parents (aget conf base ip) with
+            match get_parents (aget conf base ip) with
             [ Some ifam ->
-                let cpl = coi base ifam in
-                next_branch_same_len conf base dist False missing (mother cpl)
-                  Female ipl
+                let cpl = foi base ifam in
+                next_branch_same_len conf base dist False missing
+                  (get_mother cpl) Female ipl
             | _ -> failwith "next_branch_same_len" ]
         | Neuter -> assert False ] ]
-  else if missing == 0 then Some (ia, sa, ipl)
+  else if missing = 0 then Some (ia, sa, ipl)
   else if missing < fst dist ia || missing > snd dist ia then
     next_branch_same_len conf base dist True missing ia sa ipl
   else
-    match parents (aget conf base ia) with
+    match get_parents (aget conf base ia) with
     [ Some ifam ->
-        let cpl = coi base ifam in
-        next_branch_same_len conf base dist False (missing - 1) (father cpl) Male
-          [(ia, sa) :: ipl]
+        let cpl = foi base ifam in
+        next_branch_same_len conf base dist False (missing - 1)
+          (get_father cpl) Male [(ia, sa) :: ipl]
     | None -> next_branch_same_len conf base dist True missing ia sa ipl ]
 ;
 
 value find_next_branch conf base dist ia sa ipl =
   loop ia sa ipl where rec loop ia1 sa1 ipl =
     match next_branch_same_len conf base dist True 0 ia1 sa1 ipl with
-    [ Some (ia1, sa1, ipl) -> if ia == ia1 then Some ipl else loop ia1 sa1 ipl
+    [ Some (ia1, sa1, ipl) -> if ia = ia1 then Some ipl else loop ia1 sa1 ipl
     | _ -> None ]
 ;
 
@@ -156,29 +156,29 @@ value rec prev_branch_same_len conf base dist backward missing ia sa ipl =
         [ Male ->
             prev_branch_same_len conf base dist True (missing + 1) ip sp ipl1
         | Female ->
-            match parents (aget conf base ip) with
+            match get_parents (aget conf base ip) with
             [ Some ifam ->
-                let cpl = coi base ifam in
-                prev_branch_same_len conf base dist False missing (father cpl)
-                  Male ipl
+                let cpl = foi base ifam in
+                prev_branch_same_len conf base dist False missing
+                  (get_father cpl) Male ipl
             | _ -> failwith "prev_branch_same_len" ]
         | Neuter -> assert False ] ]
-  else if missing == 0 then Some (ia, sa, ipl)
+  else if missing = 0 then Some (ia, sa, ipl)
   else if missing < fst dist ia || missing > snd dist ia then
     prev_branch_same_len conf base dist True missing ia sa ipl
   else
-    match parents (aget conf base ia) with
+    match get_parents (aget conf base ia) with
     [ Some ifam ->
-        let cpl = coi base ifam in
-        prev_branch_same_len conf base dist False (missing - 1) (mother cpl)
-          Female [(ia, sa) :: ipl]
+        let cpl = foi base ifam in
+        prev_branch_same_len conf base dist False (missing - 1)
+          (get_mother cpl) Female [(ia, sa) :: ipl]
     | None -> prev_branch_same_len conf base dist True missing ia sa ipl ]
 ;
 
 value find_prev_branch conf base dist ia sa ipl =
   loop ia sa ipl where rec loop ia1 sa1 ipl =
     match prev_branch_same_len conf base dist True 0 ia1 sa1 ipl with
-    [ Some (ia1, sa1, ipl) -> if ia == ia1 then Some ipl else loop ia1 sa1 ipl
+    [ Some (ia1, sa1, ipl) -> if ia = ia1 then Some ipl else loop ia1 sa1 ipl
     | _ -> None ]
 ;
 
@@ -193,20 +193,24 @@ value spouse_text conf base end_sp ip ipl =
   match (ipl, (p_getenv conf.env "spouse", p_getenv conf.env "opt")) with
   [ ([(ips, _) :: _], (Some "on", _) | (_, Some "spouse")) ->
       let a = aget conf base ips in
-      match parents a with
+      match get_parents a with
       [ Some ifam ->
-          let c = coi base ifam in
           let fam = foi base ifam in
-          let sp = if ip = (father c) then (mother c) else (father c) in
+          let sp =
+            if ip = get_father fam then get_mother fam else get_father fam
+          in
           let d =
             Date.short_marriage_date_text conf base fam
-              (pget conf base (father c)) (pget conf base (mother c))
+              (pget conf base (get_father fam))
+              (pget conf base (get_mother fam))
           in
           (someone_text conf base sp, d, Some sp)
       | _ -> ("", "", None) ]
   | ([], _) ->
       match end_sp with
-      [ Some p -> (someone_text conf base p.cle_index, "", None)
+      [ Some p ->
+          (someone_text conf base (get_key_index p), "",
+           Some (get_key_index p))
       | _ -> ("", "", None) ]
   | _ -> ("", "", None) ]
 ;
@@ -215,7 +219,7 @@ value print_someone_and_spouse conf base info in_tab ip n ipl =
   let (s, d, spo) = spouse_text conf base n ip ipl in
   do {
     if in_tab && (info.bd > 0 || info.td_prop <> "") then
-      Wserver.wprint "<table border=%d><tr><td align=center%s>" info.bd
+      Wserver.wprint "<table border=\"%d\"><tr><td align=\"center\"%s>" info.bd
         info.td_prop
     else ();
     Wserver.wprint "%s\n" (someone_text conf base ip);
@@ -249,29 +253,29 @@ value rec print_both_branches conf base info pl1 pl2 =
       [ [(p2, _) :: pl2] -> (Some p2, pl2)
       | [] -> (None, []) ]
     in
-    tag "tr" "align=left" begin
-      stag "td" "align=center" begin
+    tag "tr" "align=\"%s\"" conf.left begin
+      stag "td" "align=\"center\"" begin
         match p1 with
         [ Some p1 -> Wserver.wprint "|"
         | None -> Wserver.wprint "&nbsp;" ];
       end;
       stag "td" begin Wserver.wprint "&nbsp;"; end;
-      stag "td" "align=center" begin
+      stag "td" "align=\"center\"" begin
         match p2 with
         [ Some p2 -> Wserver.wprint "|"
         | None -> Wserver.wprint "&nbsp;" ];
       end;
       Wserver.wprint "\n";
     end;
-    tag "tr" "align=left" begin
-      tag "td" "valign=top align=center width=\"50%%\"" begin
+    tag "tr" "align=\"%s\"" conf.left begin
+      tag "td" "valign=\"top\" align=\"center\"" begin
         match p1 with
         [ Some p1 ->
             print_someone_and_spouse conf base info True p1 info.sp1 pl1
         | None -> Wserver.wprint "&nbsp;" ];
       end;
       tag "td" begin Wserver.wprint "&nbsp;"; end;
-      tag "td" "valign=top align=center width=\"50%%\"" begin
+      tag "td" "valign=\"top\" align=\"center\"" begin
         match p2 with
         [ Some p2 ->
             print_someone_and_spouse conf base info True p2 info.sp2 pl2
@@ -353,7 +357,7 @@ value sign_text conf base sign info b1 b2 c1 c2 =
      | Some x -> ";td=" ^ x ]) ^
     (match p_getenv conf.env "color" with
      [ None | Some "" -> ""
-     | Some x -> ";color=" ^ x ]) ^
+     | Some x -> ";color=" ^ code_varenv x ]) ^
     include_marr conf base "3" ^ include_marr conf base "4" ^ "\">" ^ sign ^
     "</a>"
 ;
@@ -363,18 +367,20 @@ value prev_next_1_text conf base info pb nb =
     match pb with
     [ Some b1 ->
         let sign = "&lt;&lt;" in
-        sign_text conf base sign info b1 info.b2 (info.c1 - 1) info.c2 ^ " "
+        sign_text conf base sign info b1 info.b2 (info.c1 - 1) info.c2 ^ "\n"
     | _ -> "" ]
   in
   let s =
     match (pb, nb) with
     [ (None, None) -> s
-    | _ -> s ^ "<font size=-1>" ^ string_of_int info.c1 ^ "</font>" ]
+    | _ ->
+        s ^ "<span style=\"font-size:80%\">" ^ string_of_int info.c1 ^
+        "</span>" ]
   in
   match nb with
   [ Some b1 ->
       let sign = "&gt;&gt;" in
-      s ^ " " ^ sign_text conf base sign info b1 info.b2 (info.c1 + 1) info.c2
+      s ^ "\n" ^ sign_text conf base sign info b1 info.b2 (info.c1 + 1) info.c2
   | _ -> s ]
 ;
 
@@ -383,18 +389,20 @@ value prev_next_2_text conf base info pb nb =
     match pb with
     [ Some b2 ->
         let sign = "&lt;&lt;" in
-        sign_text conf base sign info info.b1 b2 info.c1 (info.c2 - 1) ^ " "
+        sign_text conf base sign info info.b1 b2 info.c1 (info.c2 - 1) ^ "\n"
     | _ -> "" ]
   in
   let s =
     match (pb, nb) with
     [ (None, None) -> s
-    | _ -> s ^ "<font size=-1>" ^ string_of_int info.c2 ^ "</font>" ]
+    | _ ->
+        s ^ "<span style=\"font-size:80%\">" ^ string_of_int info.c2 ^
+        "</span>" ]
   in
   match nb with
   [ Some b2 ->
       let sign = "&gt;&gt;" in
-      s ^ " " ^ sign_text conf base sign info info.b1 b2 info.c1 (info.c2 + 1)
+      s ^ "\n" ^ sign_text conf base sign info info.b1 b2 info.c1 (info.c2 + 1)
   | _ -> s ]
 ;
 
@@ -409,21 +417,27 @@ value print_prev_next_2 conf base info pb nb =
 value other_parent_text_if_same conf base info =
   match (info.b1, info.b2) with
   [ ([(sib1, _) :: _], [(sib2, _) :: _]) ->
-      match (parents (aget conf base sib1), parents (aget conf base sib2)) with
+      match
+        (get_parents (aget conf base sib1), get_parents (aget conf base sib2))
+      with
       [ (Some ifam1, Some ifam2) ->
-          let cpl1 = coi base ifam1 in
-          let cpl2 = coi base ifam2 in
+          let cpl1 = foi base ifam1 in
+          let cpl2 = foi base ifam2 in
           let other_parent =
-            if (father cpl1) = info.ip then
-              if (mother cpl1) = (mother cpl2) then Some (mother cpl1) else None
-            else if (father cpl1) = (father cpl2) then Some (father cpl1)
+            if get_father cpl1 = info.ip then
+              if get_mother cpl1 = get_mother cpl2 then
+                Some (get_mother cpl1)
+              else None
+            else if get_father cpl1 = get_father cpl2 then
+              Some (get_father cpl1)
             else None
           in
           match other_parent with
           [ Some ip ->
               let d =
                 Date.short_marriage_date_text conf base (foi base ifam1)
-                  (pget conf base (father cpl1)) (pget conf base (mother cpl1))
+                  (pget conf base (get_father cpl1))
+                  (pget conf base (get_mother cpl1))
               in
               Some ("&amp;" ^ d ^ " " ^ someone_text conf base ip, ip)
           | _ -> None ]
@@ -434,7 +448,7 @@ value other_parent_text_if_same conf base info =
 value print_someone_and_other_parent_if_same conf base info =
   do {
     if info.bd > 0 || info.td_prop <> "" then
-      Wserver.wprint "<table border=%d><tr><td align=center%s>"
+      Wserver.wprint "<table border=\"%d\"><tr><td align=\"center\"%s>"
         info.bd info.td_prop
     else ();
     Wserver.wprint "%s\n" (someone_text conf base info.ip);
@@ -461,7 +475,7 @@ value rec list_iter_hd_tl f =
 value print_one_branch_no_table conf base info =
   let b = if info.b1 = [] then info.b2 else info.b1 in
   let sp = if info.b1 = [] then info.sp2 else info.sp1 in
-  tag "center" begin
+  tag "div" "style=\"text-align:center\"" begin
     print_someone_and_spouse conf base info False info.ip sp b;
     html_br conf;
     list_iter_hd_tl
@@ -479,23 +493,24 @@ value print_one_branch_no_table conf base info =
 value print_one_branch_with_table conf base info =
   let b = if info.b1 = [] then info.b2 else info.b1 in
   let sp = if info.b1 = [] then info.sp2 else info.sp1 in
-  tag "table" "border=%d cellspacing=0 cellpadding=0 width=\"100%%\""
+  tag "table"
+    "border=\"%d\" cellspacing=\"0\" cellpadding=\"0\" width=\"100%%\""
     conf.border
   begin
     tag "tr" begin
-      tag "td" "align=center" begin
+      tag "td" "align=\"center\"" begin
         print_someone_and_spouse conf base info True info.ip sp b;
       end;
       list_iter_hd_tl
         (fun (ip1, _) ipl1 ->
            do {
              tag "tr" begin
-               tag "td" "align=center" begin
+               tag "td" "align=\"center\"" begin
                  Wserver.wprint "|";
                end;
              end;
              tag "tr" begin
-               tag "td" "align=center" begin
+               tag "td" "align=\"center\"" begin
                  print_someone_and_spouse conf base info True ip1 sp ipl1;
                end;
              end;
@@ -533,36 +548,34 @@ value print_two_branches_with_pre conf base info =
 ;
 
 value print_two_branches_with_table conf base info =
-  tag "table" "border=%d cellspacing=0 cellpadding=0 width=\"100%%\""
+  tag "table"
+    "border=\"%d\" cellspacing=\"0\" cellpadding=\"0\" width=\"100%%\""
     conf.border
   begin
-    tag "tr" "align=left" begin
-      stag "td" "colspan=3 align=center" begin
+    tag "tr" "align=\"%s\"" "left" begin
+      stag "td" "colspan=\"3\" align=\"center\"" begin
         print_someone_and_other_parent_if_same conf base info;
       end;
     end;
-    tag "tr" "align=left" begin
-      stag "td" "colspan=3 align=center" begin Wserver.wprint "|"; end;
+    tag "tr" "align=\"%s\"" "left" begin
+      stag "td" "colspan=\"3\" align=\"center\"" begin Wserver.wprint "|"; end;
     end;
-    tag "tr" "align=left" begin
-      stag "td" "align=right" begin
-        Wserver.wprint "<hr size=1 noshade width=\"50%%\" align=right>";
+    tag "tr" "align=\"%s\"" "left" begin
+      stagn "td" "align=\"%s\"" conf.right begin
+        xtag "hr" "dir=\"ltr\" width=\"50%%\" align=\"%s\"" conf.right;
       end;
-      Wserver.wprint "\n";
-      stag "td" begin
-        Wserver.wprint "<hr size=1 noshade width=\"100%%\">";
+      stagn "td" begin
+        xtag "hr" "width=\"100%%\"";
       end;
-      Wserver.wprint "\n";
-      stag "td" "align=left" begin
-        Wserver.wprint "<hr size=1 noshade width=\"50%%\" align=left>";
+      stagn "td" "align=\"%s\"" conf.left begin
+        xtag "hr" "dir=\"ltr\" width=\"50%%\" align=\"%s\"" conf.left;
       end;
-      Wserver.wprint "\n";
     end;
     print_both_branches conf base info info.b1 info.b2;
     if not conf.cancel_links &&
        (info.pb1 <> None || info.nb1 <> None || info.pb2 <> None ||
         info.nb2 <> None) then
-      tag "tr" "align=left" begin
+      tag "tr" "align=\"%s\"" "left" begin
         tag "td" begin
           if info.pb1 <> None || info.nb1 <> None then do {
             html_br conf; print_prev_next_1 conf base info info.pb1 info.nb1
@@ -596,7 +609,7 @@ value print_relation_path conf base info =
        (info.pb1 <> None || info.nb1 <> None || info.pb2 <> None ||
         info.nb2 <> None) then
        do {
-      html_br conf;
+      html_p conf;
       if info.pb1 <> None || info.nb1 <> None then
         print_prev_next_1 conf base info info.pb1 info.nb1
       else ();
@@ -624,6 +637,16 @@ value print_relation_ok conf base info =
     }
   in
   do {
+    let conf =
+      (* changing doctype to transitional because use of
+         <hr width=... align=...> *)
+      let doctype =
+        match p_getenv conf.base_env "doctype" with
+        [ Some ("html-4.01" | "html-4.01-trans") -> "html-4.01-trans"
+        | _ -> "xhtml-1.0-trans" ]
+      in
+      {(conf) with base_env = [("doctype", doctype) :: conf.base_env]}
+    in
     header_no_page_title conf title;
     print_relation_path conf base info;
     trailer conf
@@ -634,11 +657,11 @@ value print_relation_no_dag conf base po ip1 ip2 =
   let params =
     match (po, p_getint conf.env "l1", p_getint conf.env "l2") with
     [ (Some p, Some l1, Some l2) ->
-        let ip = p.cle_index in
+        let ip = get_key_index p in
         let dist = make_dist_tab conf base ip (max l1 l2 + 1) in
         let b1 = find_first_branch conf base dist ip l1 ip1 Neuter in
         let b2 = find_first_branch conf base dist ip l2 ip2 Neuter in
-        Some (ip, (pget conf base ip).sex, dist, b1, b2, 1, 1)
+        Some (ip, get_sex (pget conf base ip), dist, b1, b2, 1, 1)
     | _ ->
         match (p_getenv conf.env "b1", p_getenv conf.env "b2") with
         [ (Some b1str, Some b2str) ->
@@ -649,7 +672,7 @@ value print_relation_no_dag conf base po ip1 ip2 =
                branch_of_sosa conf base ip2 n2)
             with
             [ (Some [(ia1, sa1) :: b1], Some [(ia2, sa2) :: b2]) ->
-                if ia1 == ia2 then
+                if ia1 = ia2 then
                   let c1 =
                     match p_getint conf.env "c1" with
                     [ Some n -> n
@@ -679,13 +702,13 @@ value print_relation_no_dag conf base po ip1 ip2 =
         if c1 <= 1 then None else find_prev_branch conf base dist ip sp b1
       in
       let nb1 =
-        if c1 == 0 then None else find_next_branch conf base dist ip sp b1
+        if c1 = 0 then None else find_next_branch conf base dist ip sp b1
       in
       let pb2 =
         if c2 <= 1 then None else find_prev_branch conf base dist ip sp b2
       in
       let nb2 =
-        if c2 == 0 then None else find_next_branch conf base dist ip sp b2
+        if c2 = 0 then None else find_next_branch conf base dist ip sp b2
       in
       let sp1 = find_person_in_env conf base "3" in
       let sp2 = find_person_in_env conf base "4" in
@@ -695,8 +718,8 @@ value print_relation_no_dag conf base po ip1 ip2 =
         [ Some x -> " " ^ x
         | _ ->
             match Util.p_getenv conf.env "color" with
-	    [ None | Some "" -> ""
-            | Some x -> " bgcolor=" ^ x ] ]
+            [ None | Some "" -> ""
+            | Some x -> " style=\"background:" ^ x ^ "\"" ] ]
       in
       let info =
         {ip = ip; sp = sp; ip1 = ip1; ip2 = ip2; b1 = b1; b2 = b2; c1 = c1;
@@ -707,8 +730,8 @@ value print_relation_no_dag conf base po ip1 ip2 =
   | _ -> incorrect_request conf ]
 ;
 
-value print_relation_dag conf base a p1 p2 l1 l2 =
-  let ia = a.cle_index in
+value print_relation_dag conf base a ip1 ip2 l1 l2 =
+  let ia = get_key_index a in
   let add_branches dist set n ip l =
     let b = find_first_branch conf base dist ia l ip Neuter in
     let rec loop set n b =
@@ -719,7 +742,8 @@ value print_relation_dag conf base a p1 p2 l1 l2 =
             let set =
               List.fold_left (fun set (ip, _) -> Dag.Pset.add ip set) set b
             in
-            loop set (n + 1) (find_next_branch conf base dist ia a.sex b)
+            loop set (n + 1)
+              (find_next_branch conf base dist ia (get_sex a) b)
         | None -> (set, n) ]
     in
     loop set n b
@@ -731,8 +755,8 @@ value print_relation_dag conf base a p1 p2 l1 l2 =
            List.fold_left
              (fun set l2 ->
                 let dist = make_dist_tab conf base ia (max l1 l2 + 1) in
-                let (set, n) = add_branches dist set 0 p1.cle_index l1 in
-                let (set, n) = add_branches dist set n p2.cle_index l2 in
+                let (set, n) = add_branches dist set 0 ip1 l1 in
+                let (set, n) = add_branches dist set n ip2 l2 in
                 set)
              set l2)
         (Dag.Pset.add ia Dag.Pset.empty) l1
@@ -741,15 +765,22 @@ value print_relation_dag conf base a p1 p2 l1 l2 =
       List.fold_right
         (fun (ip, s) spl ->
            match find_person_in_env conf base s with
-           [ Some sp -> [(ip, (sp.cle_index, None)) :: spl]
+           [ Some sp -> [(ip, (get_key_index sp, None)) :: spl]
            | None -> spl ])
-        [(p1.cle_index, "3"); (p2.cle_index, "4")] []
+        [(ip1, "3"); (ip2, "4")] []
     in
-    let list = Dag.Pset.elements set in
-    let d = Dag.make_dag conf base list in
-    Dag.print_dag conf base set spl d
+    let elem_txt p = Dag.Item p "" in
+    let vbar_txt ip = "" in
+    let invert =
+      match Util.p_getenv conf.env "invert" with
+      [ Some "on" -> True
+      | _ -> False ]
+    in
+    let page_title = Util.capitale (Util.transl conf "tree") in
+    Dag.make_and_print_dag conf base elem_txt vbar_txt invert set spl
+      page_title ""
   with
-  [ Exit -> Util.incorrect_request conf ]
+  [ Exit -> Hutil.incorrect_request conf ]
 ;
 
 value int_list s =
@@ -767,14 +798,18 @@ value print_relation conf base p1 p2 =
   let po = find_person_in_env conf base "" in
   match (p_getenv conf.env "dag", po, l1, l2) with
   [ (Some "on", Some p, Some l1, Some l2) ->
-      print_relation_dag conf base p p1 p2 (int_list l1) (int_list l2)
-  | _ -> print_relation_no_dag conf base po p1.cle_index p2.cle_index ]
+      print_relation_dag conf base p (get_key_index p1) (get_key_index p2)
+        (int_list l1) (int_list l2)
+  | _ ->
+      print_relation_no_dag conf base po (get_key_index p1)
+        (get_key_index p2) ]
 ;
 
-value print conf base =
+value print conf base = do {
+  Wserver.wrap_string.val := Util.xml_pretty_print;
   match
     (find_person_in_env conf base "1", find_person_in_env conf base "2")
   with
   [ (Some p1, Some p2) -> print_relation conf base p1 p2
   | _ -> incorrect_request conf ]
-;
+};
