@@ -1,5 +1,5 @@
-#!/bin/sh
-# $Id: camlp4_depend.sh,v 4.6 2001/10/03 08:55:34 ddr Exp $
+#!/bin/sh -e
+# $Id: camlp4_depend.sh,v 5.2 2006/11/16 09:08:04 ddr Exp $
 
 FILES=
 DEPARGS=
@@ -9,39 +9,40 @@ for i in $*; do
     *) DEPARGS="$DEPARGS $i";;
     esac
 done
+PR_DEP=$TOP/src/pr_dep.cmo
 
 for FILE in $FILES; do
     head -1 $FILE >/dev/null || exit 1
-    set - `head -1 $FILE`
+    set - $(head -1 $FILE)
     if test "$2" = "camlp4r" -o "$2" = "camlp4o" -o "$2" = "camlp4"; then
         COMM=$2
         shift; shift
-        ARGS=`echo $* | sed -e "s/[()*]//g"`
+        ARGS=$(echo $* | sed -e "s/[()*]//g")
     else
         COMM=camlp4r
         ARGS=
     fi
     ARGS2="$DEPARGS"
-    echo $COMM pr_depend.cmo pa_ifdef.cmo $ARGS -- $ARGS2 $FILE >&2
-    $COMM pr_depend.cmo pa_ifdef.cmo $ARGS -- $ARGS2 $FILE
+    echo $COMM $PR_DEP $TOP/src/ppdef.cmo $ARGS -- $ARGS2 $FILE >&2
+    $COMM $PR_DEP $TOP/src/ppdef.cmo $ARGS -- $ARGS2 $FILE
 done
 
 for FILE in $FILES; do
     head -1 $FILE >/dev/null || exit 1
-    set - `head -1 $FILE`
+    set - $(head -1 $FILE)
     if test "$2" = "camlp4r" -o "$2" = "camlp4o" -o "$2" = "camlp4"; then
         COMM=$2
 	shift; shift
-        ARGS=`echo $* | sed -e "s/[()*]//g"`
+        ARGS=$(echo $* | sed -e "s/[()*]//g")
 	DEPS=
 	for i in $ARGS; do
-	    if test `echo $i | sed "s/^\(..\).*$/\1/"` = "./"; then
-		DEP=`echo $i | sed "s/^..\(.*\)$/\1/"`
+	    if test $(echo $i | sed "s/^\(..\).*$/\1/") = "./"; then
+		DEP=$(echo $i | sed "s/^..\(.*\)$/\1/")
 		DEPS="$DEPS $DEP"
 	    fi
 	done
         if test "$DEPS" != ""; then
-	    BASE=`basename $FILE .ml`
+	    BASE=$(basename $FILE .ml)
 	    echo $BASE.cmo $BASE.cmx: $DEPS
         fi
     fi
